<?php

namespace Clever\PaymentGateway\Gateway\Command;

use Magento\Payment\Gateway\CommandInterface;
use Magento\Sales\Model\Order;

/**
 * Class OrderCommand
 * @package Clever\PaymentGateway\Gateway\Command
 */
class OrderCommand implements CommandInterface
{
    /**
     * @var \Magento\Payment\Gateway\ConfigInterface
     */
    protected $config;

    /**
     * @var \Clever\PaymentGateway\Helper\Logger\Logger
     */
    protected $logger;

    /**
     * @var \Clever\PaymentGateway\Model\CleverPay
     */
    protected $cleverPay;

    /**
     * @var \Magento\Framework\UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var Order\Address\Renderer
     */
    protected $addressRenderer;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Store\Model\Information
     */
    protected $information;

    /**
     * @var \Magento\Theme\Block\Html\Header\Logo
     */
    protected $logo;

    /**
     * @var \Magento\Directory\Model\CountryFactory
     */
    protected $countryFactory;

    /**
     * OrderCommand constructor.
     * @param Order\Address\Renderer $addressRenderer
     * @param \Magento\Framework\UrlInterface $urlBuilder
     * @param \Clever\PaymentGateway\Model\CleverPay $cleverPay
     * @param \Magento\Payment\Gateway\ConfigInterface $config
     * @param \Clever\PaymentGateway\Helper\Logger\Logger $logger
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Store\Model\Information $information
     * @param \Magento\Theme\Block\Html\Header\Logo $logo
     * @param \Magento\Directory\Model\CountryFactory $countryFactory
     */
    public function __construct(
        \Magento\Sales\Model\Order\Address\Renderer $addressRenderer,
        \Magento\Framework\UrlInterface $urlBuilder,
        \Clever\PaymentGateway\Model\CleverPay $cleverPay,
        \Magento\Payment\Gateway\ConfigInterface $config,
        \Clever\PaymentGateway\Helper\Logger\Logger $logger,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Store\Model\Information $information,
        \Magento\Theme\Block\Html\Header\Logo $logo,
        \Magento\Directory\Model\CountryFactory $countryFactory
    ) {
        $this->config        = $config;
        $this->logger        = $logger;
        $this->cleverPay     = $cleverPay;
        $this->urlBuilder    = $urlBuilder;
        $this->addressRenderer = $addressRenderer;
        $this->storeManager  = $storeManager;
        $this->information   = $information;
        $this->logo          = $logo;
        $this->countryFactory = $countryFactory;
    }

    /**
     * @param array $commandSubject
     * @return \Magento\Payment\Gateway\Command\ResultInterface|void|null
     * @throws \Zend_Http_Client_Exception
     */
    public function execute(array $commandSubject)
    {
        try {
            $this->logger->debug('------- Begin Order Command ------------');
            $paymentDO = $commandSubject['payment'];
            $payment   = $paymentDO->getPayment();
            $payment->setSkipOrderProcessing(true);
            /** @var \Magento\Sales\Model\Order $order */
            $order     = $payment->getOrder();
            $items = [];
            foreach ($order->getAllVisibleItems() as $item) {
                if (!$item->getParentItem()) {
                    $items[] = [
                        'id' => $item->getProductId(),
                        'name' => $item->getName(),
                        'price' => $item->getPrice(),
                        'sku' => $item->getSku(),
                        'link' => $item->getProduct()->getProductUrl()
                    ];
                }
            }
            $isConnected = $this->cleverPay->checkConnection();
            /** @var \Magento\Store\Model\Store $store */
            $store = $this->storeManager->getStore();
            $storeInformation = $this->information->getStoreInformationObject($store);
            if ($isConnected['result'] ?? false) {
                $data = [
                    'order_id' => $order->getIncrementId(),
                    'payment_method' => $order->getPayment()->getMethodInstance()->getTitle(),
                    'order_details' => [
                        'id' => $order->getIncrementId(),
                        'order_key' => $order->getIncrementId(),
                        'status' => Order::STATE_PENDING_PAYMENT,
                        'date_created' => $order->getCreatedAt(),
                        'date_modified' => $order->getUpdatedAt(),
                        'total' => $order->getBaseGrandTotal(),
                        'total_tax' => $order->getTaxAmount(),
                        'shipping_total' => $order->getShippingAmount(),
                        'shipping_tax' => $order->getBaseShippingInclTax(),
                        'shipping_address' => [
                            'first_name' => $order->getShippingAddress()->getFirstname(),
                            'last_name'=> $order->getShippingAddress()->getLastname(),
                            'phone_number' => $order->getShippingAddress()->getTelephone(),
                            'email' => $order->getShippingAddress()->getEmail(),
                            'company' => $order->getShippingAddress()->getCompany(),
                            'street_address' => $order->getShippingAddress()->getStreet(),
                            'apartment' => $order->getShippingAddress()->getStreetLine(2),
                            'city' => $order->getShippingAddress()->getCity(),
                            'state' => $order->getShippingAddress()->getRegion(),
                            'postcode' => $order->getShippingAddress()->getPostcode(),
                            'country_code' => $order->getShippingAddress()->getCountryId(),
                            'country' => $this->countryFactory->create()->loadByCode($order->getShippingAddress()->getCountryId())->getName(),
                        ],
                        'billing_address' => [
                            'first_name' => $order->getBillingAddress()->getFirstname(),
                            'last_name' => $order->getBillingAddress()->getLastname(),
                            'phone_number' => $order->getBillingAddress()->getTelephone(),
                            'email' => $order->getBillingAddress()->getEmail(),
                            'company' => $order->getBillingAddress()->getCompany(),
                            'street_address' => $order->getBillingAddress()->getStreet(),
                            'apartment' => $order->getBillingAddress()->getStreet(),
                            'city' 	=> $order->getBillingAddress()->getCity(),
                            'state' => $order->getBillingAddress()->getRegion(),
                            'postcode' => $order->getBillingAddress()->getPostcode(),
                            'country_code' => $order->getBillingAddress()->getCountryId(),
                            'country' => $this->countryFactory->create()->loadByCode($order->getBillingAddress()->getCountryId())->getName(),
                        ],
                        'customer_details' => [
                            'customer_id' => $order->getCustomerId(),
                            'first_name' => $order->getCustomerFirstname(),
                            'last_name' => $order->getCustomerLastname(),
                            'email' => $order->getCustomerEmail(),
                            'address_1' => $this->addressRenderer->format($order->getBillingAddress(),'oneline'),
                            'address_2' => $this->addressRenderer->format($order->getBillingAddress(),'oneline'),
                        ],
                        'items' => $items
                    ],
                    'item_price' => $order->getBaseSubtotal(),
                    'amount' => $order->getBaseGrandTotal(),
                    /** url redirect */
                    'order_received_url' => $this->urlBuilder->getUrl('clever_pay/callback/index?order_increment='.$order->getIncrementId()),
                    'update_order_url' => $this->urlBuilder->getUrl('clever_pay/callback/index?order_increment='.$order->getIncrementId()),
                    'url_checkout' => $this->urlBuilder->getUrl('checkout'),
                    'ip' => $order->getRemoteIp(),
                    'shop' => [
                        'name' => $storeInformation->getData('name'),
                        'url' => $this->urlBuilder->getUrl('checkout/onepage/success'),
                        'icon' => $this->logo->getLogoSrc(),
                        'address' => $this->information->getFormattedAddress($store),
                        'email' => $storeInformation->getData('phone'),
                        'city' => $storeInformation->getData('city'),
                        'post_code' => $storeInformation->getData('postcode'),
                        'currency' => $this->storeManager->getStore()->getCurrentCurrencyCode(),
                        'country' => $storeInformation->getData('country'),
                    ],
                    'plugin_type'=> 'magento'
                ];
                $response = $this->cleverPay->createPayment($data);
                if ($response['redirect_url'] ?? false) {
                    $payment->setAdditionalInformation('clever_pay_url', $response['redirect_url']);
                    $order->setState(Order::STATE_PENDING_PAYMENT);
                    $order->setStatus(Order::STATE_PENDING_PAYMENT);
                    $order->setCanSendNewEmailFlag(false);
                } else {
                    throw new \Exception(__('Could not found CleverPay redirect url.'));
                }
            } else {
                throw new \Exception(__('Error while connect to CleverPay. Please check your connection!'));
            }
        } catch (\Exception $e) {
            $this->logger->debug('Order Command Error: ' . $e->getMessage());
            throw $e;
        }
    }
}