<?php

namespace Clever\PaymentGateway\Helper;

use Magento\Framework\App\Helper\Context;
use Magento\Store\Model\ScopeInterface;
use Clever\PaymentGateway\Api\Data\CleverPaymentInterface as Clever;
use DomainException;

/**
 * Class Configuration
 * @package Clever\PaymentGateway\Helper
 */
class Configuration extends \Magento\Framework\App\Helper\AbstractHelper
{
    const XML_CONFIG_SANDBOX = 'payment/clever_pay/is_sandbox';
    const XML_CONFIG_SECRET_KEY = 'payment/clever_pay/secret_key';
    const XML_CONFIG_APP_ID = 'payment/clever_pay/app_id';
    const XML_CONFIG_IS_ACTIVE = 'payment/clever_pay/active';
    const SUPPORT_METHOD = [
        'HS256' => ['hash_hmac', 'SHA256'],
        'HS512' => ['hash_hmac', 'SHA512'],
        'HS384' => ['hash_hmac', 'SHA384'],
        'RS256' => ['openssl', 'SHA256']
    ];

    /**
     * @var \Magento\Framework\Encryption\EncryptorInterface
     */
    protected $encryptor;

    /**
     * Configuration constructor.
     * @param Context $context
     * @param \Magento\Framework\Encryption\EncryptorInterface $encryptor
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Encryption\EncryptorInterface $encryptor
    ) {
        parent::__construct($context);
        $this->encryptor = $encryptor;
    }

    /**
     * @param null $scope
     * @return bool
     */
    public function isSandboxMode($scope = null)
    {
        return $this->scopeConfig->isSetFlag(
            self::XML_CONFIG_SANDBOX,
            ScopeInterface::SCOPE_WEBSITE,
            $scope
        );
    }

    /**
     * @param null $scope
     * @return mixed
     */
    public function getSecretKey($scope = null)
    {
        $secret = $this->scopeConfig->getValue(
            self::XML_CONFIG_SECRET_KEY,
            ScopeInterface::SCOPE_WEBSITE,
            $scope
        );
        return $this->encryptor->decrypt($secret);
    }

    /**
     * @param null $scope
     * @return mixed
     */
    public function getAppId($scope = null)
    {
        return $this->scopeConfig->getValue(
            self::XML_CONFIG_APP_ID,
            ScopeInterface::SCOPE_WEBSITE,
            $scope
        );
    }

    /**
     * @param $path
     * @param null $scope
     * @return string
     */
    public function getUri($path, $scope = null)
    {
        if ($this->isSandboxMode($scope)) {
            return Clever::SANDBOX_URI . $path;
        }

        return Clever::PRODUCTION_URI . $path;
    }

    /**
     * @param $msg
     * @param $key
     * @param string $method
     * @return false|string
     */
    public function signNature($msg, $key, $method = 'HS256')
    {
        if (empty(self::SUPPORT_METHOD[$method])) {
            throw new DomainException('Algorithm not supported');
        }
        list($function, $algo) = self::SUPPORT_METHOD[$method];
        switch($function) {
            case 'hash_hmac':
                return hash_hmac($algo, $msg, $key, true);
            case 'openssl':
                $signature = '';
                $success = openssl_sign($msg, $signature, $key, $algo);
                if (!$success) {
                    throw new DomainException("OpenSSL unable to sign data");
                } else {
                    return $signature;
                }
        }
    }

    /**
     * @param $input
     * @return array|string|string[]
     */
    public function urlSafeB64Encode($input)
    {
        return str_replace('=', '', strtr(base64_encode($input), '+/', '-_'));
    }

    /**
     * @param $headerB64
     * @param $bodyB64
     * @param $signature
     * @param $key
     * @param string $method
     * @return bool
     */
    public function verify($headerB64, $bodyB64, $signature, $key, $method = 'HS256')
    {
        if (empty(self::SUPPORT_METHOD[$method])) {
            throw new DomainException('Algorithm not supported');
        }

        $msg = "$headerB64.$bodyB64";

        list($function, $algo) = self::SUPPORT_METHOD[$method];
        switch($function) {
            case 'openssl':
                $success = openssl_verify($msg, $signature, $key, $algo);
                if (!$success) {
                    throw new DomainException("OpenSSL unable to verify data: " . openssl_error_string());
                } else {
                    return $signature;
                }
            case 'hash_hmac':
            default:
                $hash = hash_hmac($algo, $msg, $key, true);
                $len = min(strlen($signature), strlen($hash));

                $status = 0;
                for ($i = 0; $i < $len; $i++) {
                    $status |= (ord($signature[$i]) ^ ord($hash[$i]));
                }
                $status |= (strlen($signature) ^ strlen($hash));

                return ($status === 0);
        }
    }

    /**
     * @param $jwt
     * @param null $key
     * @param bool $verify
     * @return false|mixed
     */
    public function decode($jwt, $key = null, $verify = true)
    {
        $tks = $this->split($jwt);

        if (null === ($header = json_decode($this->urlSafeB64Decode($tks['header'])))) {
            return false;
        }
        if (null === $payload = json_decode($this->urlSafeB64Decode($tks['body']))) {
            return false;
        }
        $signature = $this->urlSafeB64Decode($tks['sig']);

        if ($verify) {
            if (empty($header->alg)) {
                return false;
            }
            if (is_array($key)) {
                if (isset($header->kid)) {
                    $key = $key[$header->kid];
                } else {
                    return false;
                }
            }

            // Check the signature
            if ($key && !$this->verify($tks['header'], $tks['body'], $signature, $key, $header->alg)) {
                return false;
            }

            // Check if the nbf if it is defined. This is the time that the
            // token can actually be used. If it's not yet that time, abort.
            if (isset($payload->nbf) && $payload->nbf > time()) {
                return false;
            }

            // Check that this token has been created before 'now'. This prevents
            // using tokens that have been created for later use (and haven't
            // correctly used the nbf claim).
            if (isset($payload->iat) && $payload->iat > time()) {
                return false;
            }

            // Check if this token has expired.
            if (isset($payload->exp) && time() >= $payload->exp) {
                return false;
            }
        }

        return $payload;
    }

    /**
     * @param $input
     * @return false|string
     */
    public static function urlSafeB64Decode($input)
    {
        $remainder = strlen($input) % 4;
        if ($remainder) {
            $padlen = 4 - $remainder;
            $input .= str_repeat('=', $padlen);
        }
        return base64_decode(strtr($input, '-_', '+/'));
    }

    /**
     * @param $jwt
     * @return array|string[]
     */
    public function split($jwt)
    {
        if (is_array($jwt)) {
            return $jwt;
        }

        $tks = explode('.', $jwt);

        if (count($tks) != 3) {
            throw new \UnexpectedValueException('Wrong number of segments');
        }

        return array_combine(array('header', 'body', 'sig'), $tks);
    }

    /**
     * @return mixed
     */
    public function isActive()
    {
        return $this->scopeConfig->getValue(self::XML_CONFIG_IS_ACTIVE);
    }
}
