<?php

namespace Clever\PaymentGateway\Model;

use Magento\Framework\Webapi\Rest\Request;
use Clever\PaymentGateway\Api\Data\CleverPaymentInterface;

/**
 * Class CleverPay
 * @package Clever\PaymentGateway\Model
 */
class CleverPay implements CleverPaymentInterface
{
    /**
     * @var \Magento\Framework\HTTP\ZendClientFactory
     */
    protected $clientFactory;

    /**
     * @var \Magento\Framework\Serialize\Serializer\Json
     */
    protected $serializer;

    /**
     * @var \Clever\PaymentGateway\Helper\Configuration
     */
    protected $configuration;

    /**
     * @var \Clever\PaymentGateway\Helper\Logger\Logger
     */
    protected $cleverLogger;

    /**
     * CleverPay constructor.
     * @param \Magento\Framework\HTTP\ZendClientFactory $clientFactory
     * @param \Magento\Framework\Serialize\Serializer\Json $serializer
     * @param \Clever\PaymentGateway\Helper\Configuration $configuration
     * @param \Clever\PaymentGateway\Helper\Logger\Logger $cleverLogger
     */
    public function __construct(
        \Magento\Framework\HTTP\ZendClientFactory $clientFactory,
        \Magento\Framework\Serialize\Serializer\Json $serializer,
        \Clever\PaymentGateway\Helper\Configuration $configuration,
        \Clever\PaymentGateway\Helper\Logger\Logger $cleverLogger
    ) {
        $this->clientFactory = $clientFactory;
        $this->serializer = $serializer;
        $this->configuration = $configuration;
        $this->cleverLogger = $cleverLogger;
    }

    /**
     * @param null $scope
     * @return array|bool|float|int|mixed|string|null
     * @throws \Zend_Http_Client_Exception
     */
    public function checkConnection($scope = null)
    {
        $uri = $this->configuration->getUri(self::CHECK_CONNECTION_PATH, $scope);
        return $this->request($uri, [], Request::HTTP_METHOD_GET);
    }

    /**
     * @param $params
     * @return array|bool|float|int|mixed|string|null
     * @throws \Zend_Http_Client_Exception
     */
    public function createPayment($params)
    {
        $data = ['info' => $this->prepareParams($params, $this->configuration->getSecretKey())];
        $uri = $this->configuration->getUri(self::CREATE_PAYMENT_PATH);
        return $this->request($uri, $data, Request::HTTP_METHOD_POST);
    }

    /**
     * @param $orderId
     * @return array|bool|float|int|mixed|string|null
     * @throws \Zend_Http_Client_Exception
     */
    public function confirmPayment($orderId)
    {
        $uri = $this->configuration->getUri(self::CONFIRM_PAYMENT_PATH) . $orderId;
        return $this->request($uri, [], Request::HTTP_METHOD_GET);
    }

    /**
     * @param $payload
     * @param $key
     * @param string $algo
     * @param null $keyId
     * @return string
     */
    public function prepareParams($payload, $key, $algo = 'HS256', $keyId = null)
    {
        $header = ['typ' => 'JWT', 'alg' => $algo];
        if ($keyId !== null) {
            $header['kid'] = $keyId;
        }
        $segments = array();
        $segments[] = $this->configuration->urlSafeB64Encode($this->serializer->serialize($header));
        $segments[] = $this->configuration->urlSafeB64Encode($this->serializer->serialize($payload));
        $signing_input = implode('.', $segments);

        $signature = $this->configuration->signNature($signing_input, $key, $algo);
        $segments[] = $this->configuration->urlSafeB64Encode($signature);

        return implode('.', $segments);
    }

    /**
     * @return array|bool|float|int|mixed|string|null
     * @throws \Zend_Http_Client_Exception
     */
    public function getCashBack()
    {
        $uri = $this->configuration->getUri(self::CASHBACK_PATH);
        return $this->request($uri, [], Request::HTTP_METHOD_GET);
    }

    /**
     * @param $uri
     * @param $params
     * @param $method
     * @return array|bool|float|int|mixed|string|null
     * @throws \Zend_Http_Client_Exception
     */
    private function request($uri, $params, $method)
    {
        try {
            $client = $this->clientFactory->create();
            $client->setUri($uri);
            if (!empty($params)) {
                $client->setRawData($this->serializer->serialize($params));
            }
            $client->setUrlEncodeBody(true);
            $client->setHeaders([
                'Content-Type'  => 'application/json',
                'secret-key' => $this->configuration->getSecretKey(),
                'app-id' => $this->configuration->getAppId()
            ]);
            $client->setMethod($method);
            $result = $client->request();

            $this->cleverLogger->debug('-------------(' . $uri . ')--------------');
            $this->cleverLogger->debug('Params: ' . $this->serializer->serialize($params));
            $this->cleverLogger->debug('Response: ' . $result->getBody());

            return $this->serializer->unserialize($result->getBody());
        } catch (\Exception $exception) {
            $this->cleverLogger->debug('Response: ' . $exception->getMessage());
            return [];
        }
    }
}
